@@ -18,9 +18,6 @@ filescanner_init(void);
 void
 filescanner_deinit(void);
 
-int
-filescanner_status(void);
-
 void
 filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct media_file_info *external_mfi);
 
@@ -32,7 +29,7 @@ int
 scan_metadata_icy(char *url, struct media_file_info *mfi);
 
 void
-scan_m3u_playlist(char *file, time_t mtime);
+scan_playlist(char *file, time_t mtime);
 
 #ifdef ITUNES
 void
