@@ -21,4 +21,7 @@ daap_request(struct evhttp_request *req);
 int
 daap_is_request(struct evhttp_request *req, char *uri);
 
+struct evbuffer *
+daap_reply_build(char *full_uri, const char *ua);
+
 #endif /* !__HTTPD_DAAP_H__ */
