@@ -1417,6 +1417,8 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
   int ret;
   int quirkyquery;
 
+  mode = 1;
+
   param = evhttp_find_header(query, "mode");
   if (param)
     {
@@ -1447,8 +1449,8 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
 	sort = "album";
       }
 
-      // only use queryfilter if mode is not equal 3 (play next) or 5 (add to up next)
-      queuefilter = (mode == 3 || mode == 5) ? NULL : evhttp_find_header(query, "queuefilter");
+      // only use queryfilter if mode is not equal 0 (add to up next), 3 (play next) or 5 (add to up next)
+      queuefilter = (mode == 0 || mode == 3 || mode == 5) ? NULL : evhttp_find_header(query, "queuefilter");
 
       querymodifier = evhttp_find_header(query, "query-modifier");
       if (!querymodifier || (strcmp(querymodifier, "containers") != 0))
@@ -2104,7 +2106,7 @@ dacp_reply_setspeakers(struct evhttp_request *req, struct evbuffer *evbuf, char
 	}
       else
 	{
-	  DPRINTF(E_DBG, L_DACP, "Speaker id converted with ret %d, param %s, dec val %llu.\n", ret, param, ids[i]);
+	  DPRINTF(E_DBG, L_DACP, "Speaker id converted with ret %d, param %s, dec val %" PRIu64 ".\n", ret, param, ids[i]);
 	}
       i++;
     }
