@@ -27,8 +27,10 @@
 #define L_DMAP    18
 #define L_DBPERF  19
 #define L_SPOTIFY 20
+#define L_LASTFM  21
+#define L_DCACHE  22
 
-#define N_LOGDOMAINS  21
+#define N_LOGDOMAINS  23
 
 /* Severities */
 #define E_FATAL   0
