@@ -58,6 +58,7 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include "db.h"
 #include "logger.h"
 #include "misc.h"
+#include "daap_cache.h"
 #include "filescanner.h"
 #include "httpd.h"
 #include "mdns.h"
@@ -67,6 +68,9 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 # include "ffmpeg_url_evbuffer.h"
 #endif
 
+#ifdef LASTFM
+# include "lastfm.h"
+#endif
 #ifdef HAVE_SPOTIFY_H
 # include "spotify.h"
 #endif
@@ -671,6 +675,16 @@ main(int argc, char **argv)
       goto db_fail;
     }
 
+  /* Spawn DAAP cache thread */
+  ret = daapcache_init();
+  if (ret != 0)
+    {
+      DPRINTF(E_FATAL, L_MAIN, "DAAP cache thread failed to start\n");
+
+      ret = EXIT_FAILURE;
+      goto cache_fail;
+    }
+
   /* Spawn file scanner thread */
   ret = filescanner_init();
   if (ret != 0)
@@ -798,6 +812,10 @@ main(int argc, char **argv)
   player_deinit();
 
  player_fail:
+#ifdef LASTFM
+  DPRINTF(E_LOG, L_MAIN, "LastFM deinit\n");
+  lastfm_deinit();
+#endif
 #ifdef HAVE_SPOTIFY_H
   DPRINTF(E_LOG, L_MAIN, "Spotify deinit\n");
   spotify_deinit();
@@ -806,9 +824,14 @@ main(int argc, char **argv)
   filescanner_deinit();
 
  filescanner_fail:
+  DPRINTF(E_LOG, L_MAIN, "DAAP cache deinit\n");
+  daapcache_deinit();
+
+ cache_fail:
   DPRINTF(E_LOG, L_MAIN, "Database deinit\n");
   db_perthread_deinit();
   db_deinit();
+
  db_fail:
   if (ret == EXIT_FAILURE)
     {
