@@ -34,17 +34,17 @@
 #if defined(__linux__)
 # include <sys/timerfd.h>
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-# include <sys/time.h>
-# include <sys/event.h>
+# include <signal.h>
 #endif
 
-#if defined(HAVE_SYS_EVENTFD_H) && defined(HAVE_EVENTFD)
-# define USE_EVENTFD
-# include <sys/eventfd.h>
+#ifdef HAVE_LIBEVENT2
+# include <event2/event.h>
+# include <event2/buffer.h>
+#else
+# include <event.h>
+# define evbuffer_get_length(x) (x)->off
 #endif
 
-#include <event.h>
-
 #include <gcrypt.h>
 
 #include "db.h"
@@ -58,6 +58,10 @@
 #include "raop.h"
 #include "laudio.h"
 
+#ifdef LASTFM
+# include "lastfm.h"
+#endif
+
 /* These handle getting the media data */
 #include "transcode.h"
 #include "pipe.h"
@@ -137,15 +141,11 @@ static const char *raop_devtype[] =
 
 struct event_base *evbase_player;
 
-#ifdef USE_EVENTFD
-static int exit_efd;
-#else
 static int exit_pipe[2];
-#endif
 static int cmd_pipe[2];
 static int player_exit;
-static struct event exitev;
-static struct event cmdev;
+static struct event *exitev;
+static struct event *cmdev;
 static pthread_t tid_player;
 
 /* Player status */
@@ -157,15 +157,17 @@ static char shuffle;
 static player_status_handler update_handler;
 
 /* Playback timer */
-static int pb_timer_fd;
-static struct event pb_timer_ev;
 #if defined(__linux__)
+static int pb_timer_fd;
+#else
+timer_t pb_timer;
+#endif
+static struct event *pb_timer_ev;
 static struct timespec pb_timer_last;
 static struct timespec packet_timer_last;
 static uint64_t MINIMUM_STREAM_PERIOD;
 static struct timespec packet_time = { 0, AIRTUNES_V2_STREAM_PERIOD };
 static struct timespec timer_res;
-#endif
 
 /* Sync source */
 static enum player_sync_source pb_sync_source;
@@ -218,7 +220,7 @@ command_async_end(struct player_command *cmd)
   pthread_mutex_unlock(&cmd->lck);
 
   /* Process commands again */
-  event_add(&cmdev, NULL);
+  event_add(cmdev, NULL);
 }
 
 static void
@@ -375,11 +377,7 @@ player_get_current_pos_clock(uint64_t *pos, struct timespec *ts, int commit)
   uint64_t delta;
   int ret;
 
-#if defined(__linux__)
   ret = clock_gettime_with_res(CLOCK_MONOTONIC, ts, &timer_res);
-#else
-  ret = clock_gettime(CLOCK_MONOTONIC, ts);
-#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_PLAYER, "Couldn't get clock: %s\n", strerror(errno));
@@ -423,11 +421,7 @@ player_get_current_pos_laudio(uint64_t *pos, struct timespec *ts, int commit)
 
   *pos = laudio_get_pos();
 
-#if defined(__linux__)
   ret = clock_gettime_with_res(CLOCK_MONOTONIC, ts, &timer_res);
-#else
-  ret = clock_gettime(CLOCK_MONOTONIC, ts);
-#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_PLAYER, "Couldn't get clock: %s\n", strerror(errno));
@@ -465,10 +459,78 @@ player_get_current_pos(uint64_t *pos, struct timespec *ts, int commit)
   return -1;
 }
 
+static int
+pb_timer_start(struct timespec *ts)
+{
+  struct itimerspec next;
+  int ret;
+
+  next.it_interval.tv_sec = 0;
+  next.it_interval.tv_nsec = 0;
+  next.it_value.tv_sec = ts->tv_sec;
+  next.it_value.tv_nsec = ts->tv_nsec;
+
+#if defined(__linux__)
+  ret = timerfd_settime(pb_timer_fd, TFD_TIMER_ABSTIME, &next, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not arm playback timer: %s\n", strerror(errno));
+
+      return -1;
+    }
+
+  ret = event_add(pb_timer_ev, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not add playback timer event\n");
+
+      return -1;
+    }
+#else
+  ret = timer_settime(pb_timer, TIMER_ABSTIME, &next, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not arm playback timer: %s\n", strerror(errno));
+
+      return -1;
+    }
+#endif
+
+  return 0;
+}
+
+static int
+pb_timer_stop(void)
+{
+  struct itimerspec next;
+  int ret;
+
+  memset(&next, 0, sizeof(struct itimerspec));
+
+#if defined(__linux__)
+  ret = timerfd_settime(pb_timer_fd, TFD_TIMER_ABSTIME, &next, NULL);
+
+  event_del(pb_timer_ev);
+#else
+  ret = timer_settime(pb_timer, TIMER_ABSTIME, &next, NULL);
+#endif
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not disarm playback timer: %s\n", strerror(errno));
+
+      return -1;
+    }
+
+  return 0;
+}
+
 /* Forward */
 static void
 playback_abort(void);
 
+static int
+queue_clear(struct player_command *cmd);
+
 static void
 player_laudio_status_cb(enum laudio_state status)
 {
@@ -655,6 +717,8 @@ player_queue_make(struct query_params *qp, const char *sort)
 
   q_head->pl_prev = q_tail;
   q_tail->pl_next = q_head;
+  q_head->shuffle_prev = q_tail;
+  q_tail->shuffle_next = q_head;
 
   return q_head;
 }
@@ -821,9 +885,12 @@ player_queue_make_daap(struct player_source **head, const char *query, const cha
 	}
       else
 	{
-	  DPRINTF(E_LOG, L_PLAYER, "Unknown queuefilter: %s\n", queuefilter);
+	  DPRINTF(E_LOG, L_PLAYER, "Unknown queuefilter %s\n", queuefilter);
 
-	  return -1;
+	  // If the queuefilter is unkown, ignore it and use the query parameter instead to build the sql query
+	  id = 0;
+	  qp.type = Q_ITEMS;
+	  qp.filter = daap_query_parse_sql(query);
 	}
     }
   else
@@ -971,8 +1038,11 @@ source_stop(struct player_source *ps)
     }
 }
 
-static struct player_source *
-source_shuffle(struct player_source *head)
+/*
+ * Shuffles the items between head and tail (excluding head and tail)
+ */
+static void
+source_shuffle(struct player_source *head, struct player_source *tail)
 {
   struct player_source *ps;
   struct player_source **ps_array;
@@ -980,34 +1050,55 @@ source_shuffle(struct player_source *head)
   int i;
 
   if (!head)
-    return NULL;
+    return;
 
-  ps = head;
+  if (!tail)
+    return;
+
+  if (!shuffle)
+    {
+      ps = head;
+      do
+	{
+	  ps->shuffle_next = ps->pl_next;
+	  ps->shuffle_prev = ps->pl_prev;
+	  ps = ps->pl_next;
+	}
+      while (ps != head);
+    }
+
+  // Count items in queue (excluding head and tail)
+  ps = head->shuffle_next;
   nitems = 0;
-  do
+  while (ps != tail)
     {
       nitems++;
-      ps = ps->pl_next;
+      ps = ps->shuffle_next;
     }
-  while (ps != head);
 
+  // Do not reshuffle queue with one item
+  if (nitems < 1)
+    return;
+
+  // Construct array for number of items in queue
   ps_array = (struct player_source **)malloc(nitems * sizeof(struct player_source *));
   if (!ps_array)
     {
       DPRINTF(E_LOG, L_PLAYER, "Could not allocate memory for shuffle array\n");
-      return NULL;
+      return;
     }
 
-  ps = head;
+  // Fill array with items in queue (excluding head and tail)
+  ps = head->shuffle_next;
   i = 0;
   do
     {
       ps_array[i] = ps;
 
-      ps = ps->pl_next;
+      ps = ps->shuffle_next;
       i++;
     }
-  while (ps != head);
+  while (ps != tail);
 
   shuffle_ptr(&shuffle_rng, (void **)ps_array, nitems);
 
@@ -1022,29 +1113,39 @@ source_shuffle(struct player_source *head)
 	ps->shuffle_next = ps_array[i + 1];
     }
 
-  ps_array[0]->shuffle_prev = ps_array[nitems - 1];
-  ps_array[nitems - 1]->shuffle_next = ps_array[0];
-
-  ps = ps_array[0];
+  // Insert shuffled items between head and tail
+  ps_array[0]->shuffle_prev = head;
+  ps_array[nitems - 1]->shuffle_next = tail;
+  head->shuffle_next = ps_array[0];
+  tail->shuffle_prev = ps_array[nitems - 1];
 
   free(ps_array);
 
-  return ps;
+  return;
 }
 
 static void
 source_reshuffle(void)
 {
-  struct player_source *ps;
-
-  ps = source_shuffle(source_head);
-  if (!ps)
-    return;
+  struct player_source *head;
+  struct player_source *tail;
 
   if (cur_streaming)
-    shuffle_head = cur_streaming;
+    head = cur_streaming;
   else
-    shuffle_head = ps;
+    head = source_head;
+
+  if (repeat == REPEAT_ALL)
+    tail = head;
+  else if (shuffle)
+    tail = shuffle_head;
+  else
+    tail = source_head;
+
+  source_shuffle(head, tail);
+
+  if (repeat == REPEAT_ALL)
+    shuffle_head = head;
 }
 
 /* Helper */
@@ -1395,6 +1496,9 @@ source_check(void)
       i++;
 
       db_file_inc_playcount((int)cur_playing->id);
+#ifdef LASTFM
+      lastfm_scrobble((int)cur_playing->id);
+#endif
 
       /* Stop playback if:
        * - at end of playlist (NULL)
@@ -1499,7 +1603,7 @@ source_read(uint8_t *buf, int len, uint64_t rtptime)
 	    return -1;
 	}
 
-      if (EVBUFFER_LENGTH(audio_buf) == 0)
+      if (evbuffer_get_length(audio_buf) == 0)
 	{
 	  switch (cur_streaming->type)
 	    {
@@ -1541,7 +1645,7 @@ source_read(uint8_t *buf, int len, uint64_t rtptime)
 static void
 playback_write(void)
 {
-  uint8_t rawbuf[AIRTUNES_V2_PACKET_SAMPLES * 2 * 2];
+  uint8_t rawbuf[STOB(AIRTUNES_V2_PACKET_SAMPLES)];
   int ret;
 
   source_check();
@@ -1569,19 +1673,19 @@ playback_write(void)
     raop_v2_write(rawbuf, last_rtptime);
 }
 
-#if defined(__linux__)
 static void
 player_playback_cb(int fd, short what, void *arg)
 {
-  struct itimerspec next;
-  uint64_t ticks;
-  int ret;
   uint32_t packet_send_count = 0;
   struct timespec next_tick;
   struct timespec stream_period  = { 0, MINIMUM_STREAM_PERIOD };
+  int ret;
+#if defined(__linux__)
+  uint64_t ticks;
 
   /* Acknowledge timer */
   read(fd, &ticks, sizeof(ticks));
+#endif /* __linux__ */
 
   /* Decide how many packets to send */
   next_tick = timespec_add(pb_timer_last, stream_period);
@@ -1599,80 +1703,18 @@ player_playback_cb(int fd, short what, void *arg)
 	  return;
 	}
     }
-
-  while(timespec_cmp(packet_timer_last, next_tick) < 0);
+  while (timespec_cmp(packet_timer_last, next_tick) < 0);
 
   /* Make sure playback is still running */
   if (player_state == PLAY_STOPPED)
     return;
 
-  pb_timer_last.tv_nsec += MINIMUM_STREAM_PERIOD;
-  if (pb_timer_last.tv_nsec >= 1000000000)
-    {
-      pb_timer_last.tv_sec++;
-      pb_timer_last.tv_nsec -= 1000000000;
-    }
+  pb_timer_last = timespec_add(pb_timer_last, stream_period);
 
-  next.it_interval.tv_sec = 0;
-  next.it_interval.tv_nsec = 0;
-  next.it_value.tv_sec = pb_timer_last.tv_sec;
-  next.it_value.tv_nsec = pb_timer_last.tv_nsec;
-
-  ret = timerfd_settime(pb_timer_fd, TFD_TIMER_ABSTIME, &next, NULL);
+  ret = pb_timer_start(&pb_timer_last);
   if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not set playback timer: %s\n", strerror(errno));
-
-      playback_abort();
-      return;
-    }
-
-  ret = event_add(&pb_timer_ev, NULL);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not re-add playback timer event\n");
-
-      playback_abort();
-      return;
-    }
+    playback_abort();
 }
-#endif /* __linux__ */
-
-
-#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-static void
-player_playback_cb(int fd, short what, void *arg)
-{
-  struct timespec ts;
-  struct kevent kev;
-  int ret;
-
-  ts.tv_sec = 0;
-  ts.tv_nsec = 0;
-
-  while (kevent(pb_timer_fd, NULL, 0, &kev, 1, &ts) > 0)
-    {
-      if (kev.filter != EVFILT_TIMER)
-        continue;
-
-      playback_write();
-
-      /* Make sure playback is still running */
-      if (player_state == PLAY_STOPPED)
-	return;
-    }
-
-  ret = event_add(&pb_timer_ev, NULL);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not re-add playback timer event\n");
-
-      playback_abort();
-      return;
-    }
-}
-#endif /* __FreeBSD__ || __FreeBSD_kernel__ */
-
 
 static void
 device_free(struct raop_device *dev)
@@ -2030,24 +2072,14 @@ device_activate_cb(struct raop_device *dev, struct raop_session *rs, enum raop_s
 
   if ((player_state == PLAY_PLAYING) && (raop_sessions == 1))
     {
-#if defined(__linux__)
       ret = clock_gettime_with_res(CLOCK_MONOTONIC, &ts, &timer_res);
-#else
-      ret = clock_gettime(CLOCK_MONOTONIC, &ts);
-#endif
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_PLAYER, "Could not get current time: %s\n", strerror(errno));
 
-#if defined(__linux__)
 	  /* Fallback to nearest timer expiration time */
 	  ts.tv_sec = pb_timer_last.tv_sec;
 	  ts.tv_nsec = pb_timer_last.tv_nsec;
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-	  if (cur_cmd->ret != -2)
-	    cur_cmd->ret = -1;
-	  goto out;
-#endif
 	}
 
       raop_playback_start(last_rtptime + AIRTUNES_V2_PACKET_SAMPLES, &ts);
@@ -2167,22 +2199,19 @@ playback_abort(void)
   if (raop_sessions > 0)
     raop_playback_stop();
 
-  if (event_initialized(&pb_timer_ev))
-    event_del(&pb_timer_ev);
-
-  if (pb_timer_fd != -1)
-    close(pb_timer_fd);
-  pb_timer_fd = -1;
+  pb_timer_stop();
 
   if (cur_playing)
     source_stop(cur_playing);
   else
     source_stop(cur_streaming);
 
+  queue_clear(NULL);
+
   cur_playing = NULL;
   cur_streaming = NULL;
 
-  evbuffer_drain(audio_buf, EVBUFFER_LENGTH(audio_buf));
+  evbuffer_drain(audio_buf, evbuffer_get_length(audio_buf));
 
   status_update(PLAY_STOPPED);
 
@@ -2300,12 +2329,7 @@ playback_stop(struct player_command *cmd)
    */
   cmd->raop_pending = raop_flush(device_command_cb, last_rtptime + AIRTUNES_V2_PACKET_SAMPLES);
 
-  if (event_initialized(&pb_timer_ev))
-    event_del(&pb_timer_ev);
-
-  if (pb_timer_fd != -1)
-    close(pb_timer_fd);
-  pb_timer_fd = -1;
+  pb_timer_stop();
 
   if (cur_playing)
     {
@@ -2321,7 +2345,7 @@ playback_stop(struct player_command *cmd)
   cur_playing = NULL;
   cur_streaming = NULL;
 
-  evbuffer_drain(audio_buf, EVBUFFER_LENGTH(audio_buf));
+  evbuffer_drain(audio_buf, evbuffer_get_length(audio_buf));
 
   status_update(PLAY_STOPPED);
 
@@ -2338,11 +2362,6 @@ playback_stop(struct player_command *cmd)
 static int
 playback_start_bh(struct player_command *cmd)
 {
-#if defined(__linux__)
-  struct itimerspec next;
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-  struct kevent kev;
-#endif
   int ret;
 
   if ((laudio_status == LAUDIO_CLOSED) && (raop_sessions == 0))
@@ -2366,11 +2385,7 @@ playback_start_bh(struct player_command *cmd)
 	}
     }
 
-#if defined(__linux__)
   ret = clock_gettime_with_res(CLOCK_MONOTONIC, &pb_pos_stamp, &timer_res);
-#else
-  ret = clock_gettime(CLOCK_MONOTONIC, &pb_pos_stamp);
-#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_PLAYER, "Couldn't get current clock: %s\n", strerror(errno));
@@ -2378,9 +2393,8 @@ playback_start_bh(struct player_command *cmd)
       goto out_fail;
     }
 
-  memset(&pb_timer_ev, 0, sizeof(struct event));
+  pb_timer_stop();
 
-#if defined(__linux__)
   /*
    * initialize the packet timer to the same relative time that we have 
    * for the playback timer.
@@ -2391,58 +2405,9 @@ playback_start_bh(struct player_command *cmd)
   pb_timer_last.tv_sec = pb_pos_stamp.tv_sec;
   pb_timer_last.tv_nsec = pb_pos_stamp.tv_nsec;
 
-  pb_timer_fd = timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC);
-  if (pb_timer_fd < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not create playback timer: %s\n", strerror(errno));
-
-      goto out_fail;
-    }
-
-  next.it_interval.tv_sec = 0;
-  next.it_interval.tv_nsec = 0;
-  next.it_value.tv_sec = pb_timer_last.tv_sec;
-  next.it_value.tv_nsec = pb_timer_last.tv_nsec;
-
-  ret = timerfd_settime(pb_timer_fd, TFD_TIMER_ABSTIME, &next, NULL);
+  ret = pb_timer_start(&pb_timer_last);
   if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not set playback timer: %s\n", strerror(errno));
-
-      goto out_fail;
-    }
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-  pb_timer_fd = kqueue();
-  if (pb_timer_fd < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not create kqueue: %s\n", strerror(errno));
-
-      goto out_fail;
-    }
-
-  memset(&kev, 0, sizeof(struct kevent));
-
-  EV_SET(&kev, 1, EVFILT_TIMER, EV_ADD | EV_ENABLE, 0, AIRTUNES_V2_STREAM_PERIOD, 0);
-
-  ret = kevent(pb_timer_fd, &kev, 1, NULL, 0, NULL);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not add kevent timer: %s\n", strerror(errno));
-
-      goto out_fail;
-    }
-#endif
-
-  event_set(&pb_timer_ev, pb_timer_fd, EV_READ, player_playback_cb, NULL);
-  event_base_set(evbase_player, &pb_timer_ev);
-
-  ret = event_add(&pb_timer_ev, NULL);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not set up playback timer event\n");
-
-      goto out_fail;
-    }
+    goto out_fail;
 
   /* Everything OK, start RAOP */
   if (raop_sessions > 0)
@@ -2453,9 +2418,6 @@ playback_start_bh(struct player_command *cmd)
   return 0;
 
  out_fail:
-  if (pb_timer_fd != -1)
-    close(pb_timer_fd);
-  pb_timer_fd = -1;
   playback_abort();
 
   return -1;
@@ -2633,6 +2595,7 @@ static int
 playback_prev_bh(struct player_command *cmd)
 {
   int ret;
+  int pos_sec;
 
   if (!cur_streaming)
     {
@@ -2646,12 +2609,34 @@ playback_prev_bh(struct player_command *cmd)
 
   source_stop(cur_streaming);
 
-  ret = source_prev();
-  if (ret < 0)
-    {
-      playback_abort();
+  /* Compute the playing time in seconds for the current song. */
+  if (cur_streaming->end > cur_streaming->stream_start)
+    pos_sec = (cur_streaming->end - cur_streaming->stream_start) / 44100;
+  else
+    pos_sec = 0;
 
-      return -1;
+  /* Only skip to the previous song if the playing time is less than 3 seconds,
+   otherwise restart the current song. */
+  DPRINTF(E_DBG, L_PLAYER, "Skipping song played %d sec\n", pos_sec);
+  if (pos_sec < 3)
+    {
+      ret = source_prev();
+      if (ret < 0)
+	{
+	  playback_abort();
+
+	  return -1;
+	}
+    }
+  else
+    {
+      ret = source_open(cur_streaming, 1);
+      if (ret < 0)
+	{
+	  playback_abort();
+
+	  return -1;
+	}
     }
 
   if (player_state == PLAY_STOPPED)
@@ -2668,6 +2653,7 @@ playback_prev_bh(struct player_command *cmd)
   return 0;
 }
 
+
 static int
 playback_next_bh(struct player_command *cmd)
 {
@@ -2846,12 +2832,7 @@ playback_pause(struct player_command *cmd)
   if (laudio_status != LAUDIO_CLOSED)
     laudio_stop();
 
-  if (event_initialized(&pb_timer_ev))
-    event_del(&pb_timer_ev);
-
-  if (pb_timer_fd != -1)
-    close(pb_timer_fd);
-  pb_timer_fd = -1;
+  pb_timer_stop();
 
   if (ps->play_next)
     source_stop(ps->play_next);
@@ -2860,7 +2841,7 @@ playback_pause(struct player_command *cmd)
   cur_streaming = ps;
   cur_streaming->play_next = NULL;
 
-  evbuffer_drain(audio_buf, EVBUFFER_LENGTH(audio_buf));
+  evbuffer_drain(audio_buf, evbuffer_get_length(audio_buf));
 
   metadata_purge();
 
@@ -3378,10 +3359,7 @@ queue_add(struct player_command *cmd)
   struct player_source *ps_tail;
 
   ps = cmd->arg.ps;
-
-  ps_shuffle = source_shuffle(ps);
-  if (!ps_shuffle)
-    ps_shuffle = ps;
+  ps_shuffle = ps;
 
   if (source_head)
     {
@@ -3411,6 +3389,9 @@ queue_add(struct player_command *cmd)
       shuffle_head = ps_shuffle;
     }
 
+  if (shuffle)
+    source_reshuffle();
+
   if (cur_plid != 0)
     cur_plid = 0;
 
@@ -3425,10 +3406,7 @@ queue_add_next(struct player_command *cmd)
   struct player_source *ps_playing;
 
   ps = cmd->arg.ps;
-
-  ps_shuffle = source_shuffle(ps);
-  if (!ps_shuffle)
-    ps_shuffle = ps;
+  ps_shuffle = ps;
 
   if (source_head && cur_streaming)
   {
@@ -3452,6 +3430,9 @@ queue_add_next(struct player_command *cmd)
     shuffle_head = ps_shuffle;
   }
 
+  if (shuffle)
+    source_reshuffle();
+
   if (cur_plid != 0)
     cur_plid = 0;
 
@@ -3708,7 +3689,7 @@ command_cb(int fd, short what, void *arg)
     }
 
  readd:
-  event_add(&cmdev, NULL);
+  event_add(cmdev, NULL);
 }
 
 
@@ -4545,8 +4526,6 @@ player_init(void)
 
   cur_cmd = NULL;
 
-  pb_timer_fd = -1;
-
   source_head = NULL;
   shuffle_head = NULL;
   cur_playing = NULL;
@@ -4559,9 +4538,8 @@ player_init(void)
 
   update_handler = NULL;
 
-  history = (struct player_history *) calloc(1, sizeof(struct player_history));
+  history = (struct player_history *)calloc(1, sizeof(struct player_history));
 
-#if defined(__linux__)
   /*
    * Determine if the resolution of the system timer is > or < the size
    * of an audio packet. NOTE: this assumes the system clock resolution
@@ -4573,9 +4551,31 @@ player_init(void)
 
       return -1;
     }
-  MINIMUM_STREAM_PERIOD = MAX(timer_res.tv_nsec, AIRTUNES_V2_STREAM_PERIOD);
+
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+  /* FreeBSD will report a resolution of 1, but actually has a resolution
+   * larger than an audio packet
+   */
+  if (timer_res.tv_nsec == 1)
+    timer_res.tv_nsec = 2 * AIRTUNES_V2_STREAM_PERIOD;
 #endif
 
+  MINIMUM_STREAM_PERIOD = MAX(timer_res.tv_nsec, AIRTUNES_V2_STREAM_PERIOD);
+
+  /* Create a timer */
+#if defined(__linux__)
+  pb_timer_fd = timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC);
+  ret = pb_timer_fd;
+#else
+  ret = timer_create(CLOCK_MONOTONIC, NULL, &pb_timer);
+#endif
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create playback timer: %s\n", strerror(errno));
+
+      return -1;
+    }
+
   /* Random RTP time start */
   gcry_randomize(&rnd, sizeof(rnd), GCRY_STRONG_RANDOM);
   last_rtptime = ((uint64_t)1 << 32) | rnd;
@@ -4593,33 +4593,22 @@ player_init(void)
     {
       DPRINTF(E_LOG, L_PLAYER, "Could not allocate evbuffer for audio buffer\n");
 
-      return -1;
+      goto audio_fail;
     }
 
   raop_v6enabled = cfg_getbool(cfg_getsec(cfg, "general"), "ipv6");
 
-
-#ifdef USE_EVENTFD
-  exit_efd = eventfd(0, EFD_CLOEXEC);
-  if (exit_efd < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not create eventfd: %s\n", strerror(errno));
-
-      goto exit_fail;
-    }
-#else
-# if defined(__linux__)
+#if defined(__linux__)
   ret = pipe2(exit_pipe, O_CLOEXEC);
-# else
+#else
   ret = pipe(exit_pipe);
-# endif
+#endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_PLAYER, "Could not create pipe: %s\n", strerror(errno));
 
       goto exit_fail;
     }
-#endif /* USE_EVENTFD */
 
 # if defined(__linux__)
   ret = pipe2(cmd_pipe, O_CLOEXEC);
@@ -4641,17 +4630,70 @@ player_init(void)
       goto evbase_fail;
     }
 
-#ifdef USE_EVENTFD
-  event_set(&exitev, exit_efd, EV_READ, exit_cb, NULL);
-#else
-  event_set(&exitev, exit_pipe[0], EV_READ, exit_cb, NULL);
-#endif
-  event_base_set(evbase_player, &exitev);
-  event_add(&exitev, NULL);
+#ifdef HAVE_LIBEVENT2
+  exitev = event_new(evbase_player, exit_pipe[0], EV_READ, exit_cb, NULL);
+  if (!exitev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create exit event\n");
+      goto evnew_fail;
+    }
 
-  event_set(&cmdev, cmd_pipe[0], EV_READ, command_cb, NULL);
-  event_base_set(evbase_player, &cmdev);
-  event_add(&cmdev, NULL);
+  cmdev = event_new(evbase_player, cmd_pipe[0], EV_READ, command_cb, NULL);
+  if (!cmdev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create cmd event\n");
+      goto evnew_fail;
+    }
+
+# if defined(__linux__)
+  pb_timer_ev = event_new(evbase_player, pb_timer_fd, EV_READ, player_playback_cb, NULL);
+# else
+  pb_timer_ev = evsignal_new(evbase_player, SIGALRM, player_playback_cb, NULL);
+# endif
+  if (!pb_timer_ev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create playback timer event\n");
+      goto evnew_fail;
+    }
+#else
+  exitev = (struct event *)malloc(sizeof(struct event));
+  if (!exitev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create exit event\n");
+      goto evnew_fail;
+    }
+  event_set(exitev, exit_pipe[0], EV_READ, exit_cb, NULL);
+  event_base_set(evbase_player, exitev);
+
+  cmdev = (struct event *)malloc(sizeof(struct event));
+  if (!cmdev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create cmd event\n");
+      goto evnew_fail;
+    }
+  event_set(cmdev, cmd_pipe[0], EV_READ, command_cb, NULL);
+  event_base_set(evbase_player, cmdev);
+
+  pb_timer_ev = (struct event *)malloc(sizeof(struct event));
+  if (!pb_timer_ev)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not create playback timer event\n");
+      goto evnew_fail;
+    }
+# if defined(__linux__)
+  event_set(pb_timer_ev, pb_timer_fd, EV_READ, player_playback_cb, NULL);
+# else
+  signal_set(pb_timer_ev, SIGALRM, player_playback_cb, NULL);
+# endif
+  event_base_set(evbase_player, pb_timer_ev);
+#endif /* HAVE_LIBEVENT2 */
+
+  event_add(exitev, NULL);
+  event_add(cmdev, NULL);
+
+#ifndef __linux__
+  event_add(pb_timer_ev, NULL);
+#endif
 
   ret = laudio_init(player_laudio_status_cb);
   if (ret < 0)
@@ -4698,19 +4740,22 @@ player_init(void)
  raop_fail:
   laudio_deinit();
  laudio_fail:
+ evnew_fail:
   event_base_free(evbase_player);
  evbase_fail:
   close(cmd_pipe[0]);
   close(cmd_pipe[1]);
  cmd_fail:
-#ifdef USE_EVENTFD
-  close(exit_efd);
-#else
   close(exit_pipe[0]);
   close(exit_pipe[1]);
-#endif
  exit_fail:
   evbuffer_free(audio_buf);
+ audio_fail:
+#if defined(__linux__)
+  close(pb_timer_fd);
+#else
+  timer_delete(pb_timer);
+#endif
 
   return -1;
 }
@@ -4720,26 +4765,15 @@ void
 player_deinit(void)
 {
   int ret;
-
-#ifdef USE_EVENTFD
-  ret = eventfd_write(exit_efd, 1);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_PLAYER, "Could not send exit event: %s\n", strerror(errno));
-
-      return;
-    }
-#else
   int dummy = 42;
 
   ret = write(exit_pipe[1], &dummy, sizeof(dummy));
   if (ret != sizeof(dummy))
     {
-      DPRINTF(E_LOG, L_PLAYER, "Could not write to exit fd: %s\n", strerror(errno));
+      DPRINTF(E_LOG, L_PLAYER, "Could not write to exit pipe: %s\n", strerror(errno));
 
       return;
     }
-#endif
 
   ret = pthread_join(tid_player, NULL);
   if (ret != 0)
@@ -4754,22 +4788,20 @@ player_deinit(void)
 
   free(history);
 
+  pb_timer_stop();
+#if defined(__linux__)
+  close(pb_timer_fd);
+#else
+  timer_delete(pb_timer);
+#endif
+
   evbuffer_free(audio_buf);
 
   laudio_deinit();
   raop_deinit();
 
-  if (event_initialized(&pb_timer_ev))
-    event_del(&pb_timer_ev);
-
-  event_del(&cmdev);
-
-#ifdef USE_EVENTFD
-  close(exit_efd);
-#else
   close(exit_pipe[0]);
   close(exit_pipe[1]);
-#endif
   close(cmd_pipe[0]);
   close(cmd_pipe[1]);
   cmd_pipe[0] = -1;
