@@ -4,14 +4,9 @@
 
 #include <stdint.h>
 
-#if defined(__linux__)
 /* AirTunes v2 packet interval in ns */
 /* (352 samples/packet * 1e9 ns/s) / 44100 samples/s = 7981859 ns/packet */
 # define AIRTUNES_V2_STREAM_PERIOD 7981859
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-/* AirTunes v2 packet interval in ms */
-# define AIRTUNES_V2_STREAM_PERIOD   8
-#endif
 
 /* AirTunes v2 number of samples per packet */
 #define AIRTUNES_V2_PACKET_SAMPLES  352
